{% load comment_tags %}
{% has_reacted user=user comment=comment reaction="like" as has_user_liked %}
{% block like_img_icon %}
<svg class="comment-reaction-icon reaction-like {% if has_user_liked %}user-has-reacted{% else %}user-has-not-reacted{% endif %}"
     xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 24 24"
     stroke="{% block like_icon_color %}#427297{% endblock like_icon_color %}" stroke-width="1.7" stroke-linecap="round" stroke-linejoin="round">
    <path d="M14 9V5a3 3 0 0 0-3-3l-4 9v11h11.28a2 2 0 0 0 2-1.7l1.38-9a2 2 0 0 0-2-2.3zM7 22H4a2 2 0 0 1-2-2v-7a2 2 0 0 1 2-2h3"></path>
</svg>
{% endblock like_img_icon %}
