<button class="{% block apply_btn_cls %}btn btn-link{% endblock apply_btn_cls %}" type="submit" name="form_type">
{% block apply_img_icon %}
<svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24"
        fill="none" stroke="{% block apply_icon_color %}#00bc8c{% endblock apply_icon_color %}" stroke-width="2" stroke-linecap="round"
        stroke-linejoin="round" class="feather feather-check">
    <polyline points="20 6 9 17 4 12"></polyline>
</svg>
{% endblock apply_img_icon %}
</button>
