# Generated by Django 3.0.5 on 2020-04-19 14:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('comment', '0002_comment_edit_date'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='comment',
            options={'ordering': ['-posted']},
        ),
        migrations.RenameField(
            model_name='comment',
            old_name='edit_date',
            new_name='edited',
        ),
        migrations.RenameField(
            model_name='comment',
            old_name='posted_date',
            new_name='posted',
        ),
    ]
