<a class="js-comment-cancel {% block cancel_btn_cls %}btn btn-link{% endblock cancel_btn_cls %}" href="#">
{% block cancel_img_icon %}
<svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24"
     fill="none" stroke="{% block cancel_icon_color %}#E74C3C{% endblock cancel_icon_color %}" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-x">
    <line x1="18" y1="6" x2="6" y2="18"></line><line x1="6" y1="6" x2="18" y2="18"></line>
</svg>
{% endblock cancel_img_icon %}
</a>
