{% load i18n %}
{% load comment_tags %}
<span title="{% if user|has_flagged:comment %}Remove flag{% else %}Report comment{% endif %}">
{% block flag_img_icon %}
<svg class="comment-flag-icon {% if user|has_flagged:comment %}user-has-flagged{% else %}user-has-not-flagged{% endif %}"
     xmlns="http://www.w3.org/2000/svg" width="15" height="15" viewBox="0 0 24 24"
     stroke="{% block flag_icon_color %}#427297{% endblock flag_icon_color %}" stroke-width="1.7" stroke-linecap="round" stroke-linejoin="round">
    <path d="M4 15s1-1 4-1 5 2 8 2 4-1 4-1V3s-1 1-4 1-5-2-8-2-4 1-4 1z"></path><line x1="4" y1="22" x2="4" y2="15"></line>
</svg>
{% endblock flag_img_icon %}
</span>
