{% load comment_tags %}
<a title="Edit comment" class="js-comment-edit {% block edit_link_cls %}btn btn-link{% endblock edit_link_cls %}"
   href="{% url 'comment:edit' pk=comment.id %}?comments_per_page={{comments_per_page}}&model_name={% get_model_name model_object %}&model_id={{ model_object.id }}&app_name={% get_app_name model_object %}&oauth={{oauth}}">
{% block edit_img_icon %}
<svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24" fill="none"
     stroke="{% block edit_icon_color %}#00bc8c{% endblock edit_icon_color %}"
     stroke-width="1.7" stroke-linecap="round" stroke-linejoin="round" class="feather feather-edit">
    <path d="M11 4H4a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h14a2 2 0 0 0 2-2v-7"></path>
    <path d="M18.5 2.5a2.121 2.121 0 0 1 3 3L12 15l-4 1 1-4 9.5-9.5z"></path>
</svg>
{% endblock edit_img_icon %}
</a>

