.js-comment-content {
    font-family: sans-serif, serif;
    font-size: 1.1rem;
}
#comments .border-bottom {
    border-bottom: 3px solid #00bc8c !important;
}
#comments .hr {
    margin-top: 0.2rem;
    margin-bottom: 2.5rem;
}
#comments .bb {
    padding-bottom: 0.3rem;
}
#comments .text-wrap {
    word-wrap: break-word;
}
#comments .h7 {
    font-size: 0.9rem;
    font-family: inherit;
    font-weight: 500;
    line-height: 1.2;
    color: inherit;
}
#comments .btn:disabled {
    cursor: not-allowed;
}
#comments textarea {
    min-height: 31px;
    padding: 4px 15px !important;
    overflow: hidden;
    resize: none;
}
#comments textarea:focus {
    border: 1px solid #00bc8c;
    outline: none;
}
.form-control:focus::placeholder,
.s-input:focus::placeholder,
input#search-input:focus::placeholder {
    opacity: 0.5;
}
#comments .js-comment-input:invalid {
    box-shadow: none;
}
#comments .js-comment-input:focus {
    box-shadow: 0 0 0 0.2rem rgba(75, 142, 213, 0.25);
}
#comments .edit-comment::before {
    content: none;
}
#comments .btn-link {
    text-decoration: none !important;
    padding: 0;
    font-size: 100%;
    font-weight: 400;
}
#comments .user-has-reacted,
.user-has-flagged {
    fill: #ffc96c;
}
#comments .user-has-not-reacted,
.user-has-not-flagged {
    fill: none;
}
.user-has-flagged,
.user-has-not-flagged {
    cursor: pointer;
}
.flag-modal textarea {
    display: none;
}

.flagged-comment {
    color: #999;
}