{% load comment_tags %}
<script>window.CSRF_TOKEN = "{{ csrf_token }}"</script>
<section id="comments" class="{% block comment_section_cls %}my-5 mx-3{% endblock comment_section_cls %}">
    <strong class="border-bottom bb"><span class="js-comment-number">{% get_comments_count model_object user %}</span> Comments</strong>
    <hr class="hr"/>
    <div class="js-comment comment mb-5">
        {% include 'comment/comments/create_comment.html' with placeholder="comment on this post..." comment=None %}
    </div>
    <div class="js-main-comment mt-5">
      {% for comment in comments %}
          {% include 'comment/comments/parent_comment.html' with placeholder='reply to this comment...' %}
      {% endfor %}
    </div>
    <div class="modal fade" id="Modal">
      <div class="modal-dialog" role="document">
        <div class="modal-content">
        </div>
    </div>
    </div>
    {% if comments.paginator.num_pages > 1 %}
    {% block pagination %}
    {% include 'comment/comments/pagination.html' with active_btn='bg-success' text_style='text-success' li_cls='page-item rounded mx-1' %}
    {% endblock pagination %}
    {% endif %}
</section>
