{% load comment_tags %}
<form class="js-comment-delete-form" method="post" data-url="{% if comment %}{% url 'comment:delete' pk=comment.id %}{% endif %}">
    {% csrf_token %}
    <div class="modal-header">
        <h5 class="modal-title">{% block title %}Confirm comment deletion{% endblock title %}</h5>
        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
            <span aria-hidden="true">&times;</span>
        </button>
    </div>
    <div class="modal-body">
        <p class="lead">{% block content %}Are you sure you want to delete this comment {% endblock content %}<strong>"{{ comment.content|slice:":25" }}"</strong>?</p>
    </div>
    <div class="modal-footer">
        <button type="button" class="{% block close_btn_cls %}btn btn-secondary{% endblock close_btn_cls %}" data-dismiss="modal">Close</button>

        <input name="model_name" value="{{model_name}}" hidden>
        <input name="model_id" value="{{model_id}}" hidden>
        <input name="app_name" value="{{app_name}}" hidden>
        <input name="comments_per_page" value="{{comments_per_page}}" hidden>
        <input name="has_parent" value="{{has_parent}}" hidden>
        <input name="oauth" value="{{oauth}}" hidden>
        <button id="modal-btn" type="submit" class="{% block del_btn_cls %}btn btn-danger{% endblock del_btn_cls %}" name={{comments_per_page}}>
            Delete
        </button>
    </div>
</form>
