# Generated by Django 3.0.5 on 2020-05-21 13:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('comment', '0004_reaction_reactioninstance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reaction',
            name='comment',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='comment.Comment'),
        ),
    ]
